------------------------------------------------------------------------------
--
--  procedure Control (body)
--
------------------------------------------------------------------------------
--  Standard packages:

with Ada.Text_IO;

------------------------------------------------------------------------------
--  Other packages:

with Base_Types;
with Landscapes;
with Server_Communications;
with Strategy;
with Tracking;

------------------------------------------------------------------------------

procedure Control is

begin --  Control
   ---------------------------------------------------------------------------
   --  Initialisation:

Initialise_Board:
   declare
      Board_Width  : Base_Types.X_Coordinate;
      Board_Height : Base_Types.Y_Coordinate;
   begin
      Server_Communications.Initialise (Width  => Board_Width,
                                        Height => Board_Height);
   Get_Board:
      declare
         use type Base_Types.X_Coordinate;
         use type Base_Types.Y_Coordinate;

         Board     : Base_Types.Board_Type (0 .. Board_Width + 1,
                                            0 .. Board_Height + 1);
      begin
         Server_Communications.Get_Board (Width  => Board_Width,
                                          Height => Board_Height,
                                          Board  => Board);
         Tracking.Set_Board (Board => Board);
         Strategy.Initialise (Board => Board);
      end Get_Board;
   end Initialise_Board;

Initialise_Robot:
   declare
      Our_Robot : Base_Types.Robot_Configuration;
   begin
      Server_Communications.Get_Robot_Configuration (Robot => Our_Robot);
      Tracking.Set_Robot_Configuration (Configuration => Our_Robot);
   end Initialise_Robot;

   ---------------------------------------------------------------------------
   --  Game play:

Game_Play:
   loop
      ------------------------------------------------------------------------
      --  What happened (3):

   Get_Actions:
      declare
         use Server_Communications;
         use Tracking;

         Affected_Robot : Base_Types.Robot_ID;
         Action         : Action_Result;
      begin
         Receive_Action_Results;
         Start_Keeping_Track_Of_Robots;

         while More_Action_Results loop
            if Action_Robot_Change then
               Get_Action_Robot_Change (Robot => Affected_Robot);
               Register_Robot_Exists (Robot => Affected_Robot);
            else
               Get_Action_Result (Robot => Affected_Robot,
                                  Item  => Action);
               Update_With (Robot  => Affected_Robot,
                            Result => Action);
            end if;
         end loop;

         Stop_Keeping_Track_Of_Robots;
      end Get_Actions;

      ------------------------------------------------------------------------
      --  Server reports available packages (1):

  Get_Available_Gifts:
      declare
         Gift_ID   : Base_Types.Gift_ID;
         Gift_Data : Base_Types.Gift_Type;
      begin
         Server_Communications.Receive_Available_Gifts;
         Tracking.Clear_Available_Gifts;
         while Server_Communications.More_Available_Gifts loop
            Server_Communications.Get_Gift_Data (ID   => Gift_ID,
                                                 Data => Gift_Data);
            Tracking.Gift_At_Tile (Gift => Gift_Data);
         end loop;
      end Get_Available_Gifts;

      ------------------------------------------------------------------------
      --  Client issues robot command (2):

  Issue_Command:
      declare
         Bid     : Base_Types.Bid_Type;
         Command : Base_Types.Command;
      begin
         Strategy.Choose_Move (Bid  => Bid,
                               Move => Command);
         Server_Communications.Request_Action (Bid  => Bid,
                                               Item => Command);
      end Issue_Command;

      ------------------------------------------------------------------------
      --  Report board state:

   Report_Board_State:
      declare
         use Ada.Text_IO;
         use Base_Types;
         use Tracking;

         Board : constant Board_Type := Return_Board_State;
      begin
         for Y in reverse Board'Range (2) loop
            for X in Board'Range (1) loop
               if Board (X, Y).Occupied then
                  if Board (X, Y).Robot = Own_Configuration.ID then
                     Put ("R");
                  else
                     Put ("r");
                  end if;
               else
                  case Board (X, Y).Tile is
                     when Water =>
                        Put ("~");
                     when Plain =>
                        Put (".");
                     when Wall  =>
                        Put ("#");
                     when Home  =>
                        Put ("@");
                  end case;
               end if;
            end loop;
            New_Line;
         end loop;
      end Report_Board_State;

      ------------------------------------------------------------------------
   end loop Game_Play;

   ---------------------------------------------------------------------------
end Control;
