------------------------------------------------------------------------------
--
--  package Landscapes (spec)
--
------------------------------------------------------------------------------
--  Standard packages:

------------------------------------------------------------------------------
--  Other packages:

with Base_Types;

------------------------------------------------------------------------------

package Landscapes is

   ---------------------------------------------------------------------------
   --  type Spot:

   type Interpolation_Class is (Ignore, Fixed, Flexible);

   type Spot (Interpolation : Interpolation_Class := Ignore) is
      record
         case Interpolation is
            when Ignore =>
               null;
            when Fixed | Flexible =>
               Height : Float;
         end case;
      end record;

   ---------------------------------------------------------------------------
   --  type Landscape:

   type Landscape is array (Base_Types.X_Coordinate range <>,
                            Base_Types.Y_Coordinate range <>) of Spot;

   type Landscape_Reference is access all Landscape;

   ---------------------------------------------------------------------------
   --  type Mapping:

   type Mapping is array (Base_Types.Tile_Types) of Spot;

   Default_Map : constant Mapping :=
     (Base_Types.Plain => (Interpolation => Flexible,
                           Height        => 10.0),
      Base_Types.Water => (Interpolation => Ignore),
      --Base_Types.Water => (Interpolation => Fixed,
      --                     Height        => -10.0),
      Base_Types.Wall  => (Interpolation => Ignore),
      Base_Types.Home  => (Interpolation => Flexible,
                           Height        => 10.0));

   ---------------------------------------------------------------------------
   --  procedure To_Landscape:

   procedure To_Landscape (Source : in     Base_Types.Board_Type;
                           Target :    out Landscape;
                           Map    : in     Mapping := Default_Map);

   ---------------------------------------------------------------------------
   --  procedure Interpolate:

   procedure Interpolate (Item       : in out Landscape;
                          Iterations : in     Positive);

   ---------------------------------------------------------------------------
   --  procedure Save:

   procedure Save (Item : in     Landscape;
                   Name : in     String);

   ---------------------------------------------------------------------------

end Landscapes;
