------------------------------------------------------------------------------
--
--  procedure Simple_Server (body)
--
--   1998-2000, cole Nationale Suprieure des Tlcommunications
--   2002, Jacob Sparre Andersen and Christian Mikkelsen
--
--  Based on the "Listener" example in the AdaSockets components.
--
-----------------------------------------------------------------------------
--                                                                         --
--                         ADASOCKETS COMPONENTS                           --
--                                                                         --
--                            L I S T E N E R                              --
--                                                                         --
--                                B o d y                                  --
--                                                                         --
--                        $ReleaseVersion: 0.1.6 $                         --
--                                                                         --
--                        Copyright (C) 1998-2000                          --
--             cole Nationale Suprieure des Tlcommunications           --
--                                                                         --
--   AdaSockets is free software; you can  redistribute it and/or modify   --
--   it  under terms of the GNU  General  Public License as published by   --
--   the Free Software Foundation; either version 2, or (at your option)   --
--   any later version.   AdaSockets is distributed  in the hope that it   --
--   will be useful, but WITHOUT ANY  WARRANTY; without even the implied   --
--   warranty of MERCHANTABILITY   or FITNESS FOR  A PARTICULAR PURPOSE.   --
--   See the GNU General Public  License  for more details.  You  should   --
--   have received a copy of the  GNU General Public License distributed   --
--   with AdaSockets; see   file COPYING.  If  not,  write  to  the Free   --
--   Software  Foundation, 59   Temple Place -   Suite  330,  Boston, MA   --
--   02111-1307, USA.                                                      --
--                                                                         --
--   As a special exception, if  other  files instantiate generics  from   --
--   this unit, or  you link this  unit with other  files to produce  an   --
--   executable,  this  unit does  not  by  itself cause  the  resulting   --
--   executable to be  covered by the  GNU General Public License.  This   --
--   exception does  not  however invalidate any  other reasons  why the   --
--   executable file might be covered by the GNU Public License.           --
--                                                                         --
--   The main repository for this software is located at:                  --
--       http://www.infres.enst.fr/ANC/                                    --
--                                                                         --
--   If you have any question, please send a mail to                       --
--       Samuel Tardieu <sam@inf.enst.fr>                                  --
--                                                                         --
-----------------------------------------------------------------------------

with Ada.Characters.Latin_1;
with Ada.Command_Line;      use Ada.Command_Line;
with Ada.Exceptions;        use Ada.Exceptions;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Text_IO;           use Ada.Text_IO;
with Sockets;               use Sockets;
with UStrings;              use UStrings;

procedure Simple_Server is

   --  Usage: listener port
   --  Example: listener 5000
   --  then telnet localhost 5000

   task type To_Stream is
      entry Start (FD : in Socket_FD);
   end To_Stream;

   task body To_Stream is
      Sock : Socket_FD;
      Line : Unbounded_String;
   begin
      select
         accept Start (FD : in Socket_FD) do
            Sock := FD;
         end Start;
      or
         terminate;
      end select;

      while not End_Of_File (Standard_Input) loop
         Get_Line (File => Standard_Input,
                   Item => Line);
         Put (Socket => Sock,
              Str    => To_String (Line) & Ada.Characters.Latin_1.LF);
      end loop;
   exception
      when Connection_Closed =>
         Put_Line ("Connection closed");
         Shutdown (Sock, Both);
   end To_Stream;

   task type From_Stream is
      entry Start (FD : in Socket_FD);
   end From_Stream;

   task body From_Stream is
      Sock : Socket_FD;
   begin
      select
         accept Start (FD : in Socket_FD) do
            Sock := FD;
         end Start;
      or
         terminate;
      end select;

      loop
         Put_Line (File => Standard_Output,
                   Item => Get_Line (Sock));
      end loop;
   exception
       when Connection_Closed =>
          Put_Line ("Connection closed");
          Shutdown (Sock, Both);
   end From_Stream;

   Accepting_Socket : Socket_FD;
   Incoming_Socket  : Socket_FD;

   Input  : To_Stream;
   Output : From_Stream;

begin
   if Argument_Count /= 1 then
      Raise_Exception (Constraint_Error'Identity,
                       "Usage: " & Command_Name & " port");
   end if;
   Socket (Accepting_Socket, AF_INET, SOCK_STREAM);
   Setsockopt (Accepting_Socket, SOL_SOCKET, SO_REUSEADDR, 1);
   Bind (Accepting_Socket, Positive'Value (Argument (1)));
   Listen (Accepting_Socket);

   Put_Line ("Waiting for new connection");
   Accept_Socket (Accepting_Socket, Incoming_Socket);
   Put_Line ("New connection acknowledged");

   --  This step is optional, especially for TCP sockets
   Set_Buffer (Incoming_Socket, 4096);
   Put_Line ("New socket buffered");

   Input.Start (Incoming_Socket);
   Output.Start (Incoming_Socket);
end Simple_Server;
